<?php
/**
 * functions.phpでは、WordPressのテーマに関するオプション機能の有効化や設定、および
 * Cafe Miraggioテーマ独自機能とカスタムテンプレートタグを定義しています。
 * 
 * Cafe Miraggioテーマは、このテーマをそのまま利用するだけでなく、子テーマを作成する際の
 * ベースとなるテーマとしやすいように子テーマでオーバーライドできるセットアップ方法や
 * 豊富な機能を備え、さらに各箇所にカスタマイズしやすい独自のフックを設けています。
 * 
 * 初心者だけでなく、中級者のWeb制作者にとってもビジネスサイトを作成する際のテーマとして
 * 利用していただければ幸いです。
 * 
 * 子テーマについて、書籍中では触れておりませんが、既に完成されたテーマを元にして、
 * 最低限の実装のみでオリジナルのテーマを開発できる機能です。
 * 優れた機能を持つテーマの機能を受け継ぎながら、HTMLのソースや表示スタイルをカスタマイズ
 * したいような場合、子テーマの機能を用いれば、テーマ開発の期間を極端に短縮できるはずです。
 * 
 * 子テーマを作るには、style.cssの冒頭に記述するコメント部分に Template という項目を記述し、
 * 親テーマを指定します。具体的な方法などはWordPress Codexを参照してください。
 * 
 * WordPress Codex 日本語版 [テーマの作成] > [子テーマの style.css]
 * http://wpdocs.sourceforge.jp/%E3%83%86%E3%83%BC%E3%83%9E%E3%81%AE%E4%BD%9C%E6%88%90#.E5.AD.90.E3.83.86.E3.83.BC.E3.83.9E.E3.81.AE_style.css
 * 
 * WordPress Codex Child Themes （英語）
 * http://codex.wordpress.org/Child_Themes
 * 
 */

/**
 * グローバル変数 $content_width を設定しておくと、ビジュアルエディター内の表示幅が制限されるようになります。
 * テーマの本文が表示される幅をピクセル数でしておけば、テキストの改行位置などビジュアルエディターでの表示が
 * Webサイトで表示したときと近くなります。
 * issetで分岐処理がなされているため、子テーマを利用する場合は子テーマで $content_width を設定しておけば、
 * 本テーマの数値は無視されます。
 * 
 */
if ( ! isset( $content_width ) ) {
	$content_width = 700;
}

/**
* テーマ関数の読み込みが終わったときに、miraggio_setup が実行されるようにアクションフックを追加します。
* after_setup_theme の実行タイミングは、「WordPressの実行フローを視覚化してみる」（ http://www.warna.info/archives/279/ ）
* wp-settings.phpの6と7の間になります。
*/
add_action( 'after_setup_theme', 'miraggio_setup' );

/**
* function_existsの分岐処理によりmiraggio_setup関数がない場合に、miraggio_setupが読み込まれます。
* これは、子テーマを利用する際に、子テーマでmiraggio_setup定義しておけば、本テーマのmiraggio_setupを
* オーバーライドできるようにするためです。
*/
if ( ! function_exists( 'miraggio_setup' ) ) :

function miraggio_setup() {
	/**
	* add_editor_styleは、ビジュアルエディターの表示にCSSを適用させるためのものです。詳細については、P181の
	* 「editor-style.cssで編集エリアをより見た目に近づける」を参照してください。
	* ビジュアルエディターにeditor-style.cssを反映させる場合、add_editor_style();のコメントアウトを外して有効化
	* してください。ただし、editor-style.cssは同梱していませんので、style.cssをコピーした上でリネームし
	* テーマとして認識させるためのコメント部分を全て削除してください。
	*/
//	add_editor_style();

	/**
	* アイキャッチ画像を利用できるようにします。
	* add_theme_supportは、パラメーターで指定したテーマ特有の機能を有効化させるWordPressの関数です。
	* 指定可能なパラメーターと機能名は現在次の通りです。
	* 
	* custom-header        : カスタムヘッダー
	* custom-background    : カスタム背景
	* post-thumbnails      : アイキャッチ画像
	* menus                : カスタムメニュー（register_nav_menus関数内で自動的にカスタムメニュー機能は有効化されます）
	* automatic-feed-links : フィードリンク
	* editor-style         : エディターのスタイル
	*                       （エディターのスタイルは、add_editor_style関数内で自動的に有効化されます。また、functions.phpでadd_theme_support( 'editor-style' );を行うことは推奨されていません）
	*/
	add_theme_support( 'post-thumbnails' );

	/*
	 * headタグ内に以下のようなフィードリンクを自動的に出力します。
	 * <link rel="alternate" type="application/rss+xml" title="Cafe Miraggio &raquo; フィード" href="http://www.cafe-miraggio.com/feed/" />
	 * <link rel="alternate" type="application/rss+xml" title="Cafe Miraggio &raquo; コメントフィード" href="http://www.cafe-miraggio.com/comments/feed/" />
	 * 参考：WordPressのヘッダーにフィードのlink要素を自動的に出力する条件（ http://www.warna.info/archives/1275/ ）
	 */ 
	add_theme_support( 'automatic-feed-links' );

	/**
	 * カスタムメニュー機能を有効化し、ナビゲーションの場所と名称を登録します。
	 * パラメーターは配列で指定し、配列の１つ１つがナビゲーションのスラッグと名前の指定となります。
	 * 各ナビゲーションの指定は、連想配列で、キーがtheme_locationのスラッグ、値が管理画面の「テーマの場所」で表示される名称です。
	 * スラッグは、wp_nav_menuのtheme_locationパラメーターでナビゲーションを特定するための値として用いられます。
	 */
	register_nav_menus( array(
		'header' => 'ヘッダーナビ',
		'global' => 'グローバルナビ',
		'footer' => 'フッターナビ',
	) );

	// デフォルトとなるヘッダー画像のパスを指定します。
	// パラメーターの%sの部分はテーマこのテーマのディレクトリの置き換えられるので、それ以降のパスを記述します。
	if ( ! defined( 'HEADER_IMAGE' ) )
		define( 'HEADER_IMAGE', '%s/img/custom-header.png' );

	// ヘッダー画像の幅と高さを定義します。Cafe Miraggioテーマでは、幅が940px、高さは340pxで設計されています。
	// ただし、サブページではCSSで上下を非表示とし、高さは240pxで表示されるようになっています。
	define( 'HEADER_IMAGE_WIDTH', apply_filters( 'miraggio_header_image_width', 940 ) );
	define( 'HEADER_IMAGE_HEIGHT', apply_filters( 'miraggio_header_image_height', 340 ) );

	/**
	 * NO_HEADER_TEXTの定義で、ヘッダーテキストの表示機能の指定を行うことができます。
	 * Cafe Miraggioテーマでは、この機能は利用しないため、trueと指定しています。
	 * ただし、機能を利用しない場合でもHEADER_TEXTCOLOR（デフォルトのヘッダーテキスト文字色）は定義しておく必要があります。
	 */
	define( 'NO_HEADER_TEXT', true );
	define( 'HEADER_TEXTCOLOR', '' );

	/* アイキャッチ画像のサイズを定義します。
	* set_post_thumbnail_sizeは、アイキャッチ画像のデフォルトであるpost-thumbnailのサイズを設定します。
	* add_image_sizeは、WordPressのデフォルトサイズ（サムネイル、中サイズ、大サイズ）以外にリサイズされる画像サイズを追加できます。
	* ただし、記事に挿入するサイズとして選択できるようになるわけではありませんので注意してくだい。
	* 参考：WordPressのアイキャッチ画像を複数サイズ表示させる（ http://www.warna.info/archives/993/ ）
	*/
	set_post_thumbnail_size( apply_filters( 'miraggio_post_thumbnail_width', 50 ), apply_filters( 'miraggio_post_thumbnail_height', 50 ), true );
	add_image_size( 'large_thumbnail', apply_filters( 'miraggio_large_thumbnail_width', 140 ), apply_filters( 'miraggio_large_thumbnail_height', 140 ), true );
	
	// ヘッダー画像の管理画面に表示用CSSを出力する関数名をパラメーターで指定します。
	// Cafe Miraggioテーマでは、適用しないため空文字列とfalseを返す関数__return_falseを指定しています。
	add_custom_image_header( '', '__return_false' );

	/*
	 * 初期選択できるヘッダー画像の登録を行います。
	 * パラメーターは配列でデフォルトの画像の数だけ指定します。
	 * 各画像のパラメーターは
	 *   url           : ヘッダー画像のパス
	 *   thumbnail_url : 管理画面で表示されるサムネイル画像のパス
	 *   description   : デフォルト画像の説明
	 * となっています。
	 */
	register_default_headers(
		array(
			'interior' => array(
				'url' => '%s/img/custom-header.png',
				'thumbnail_url' => '%s/img/custom-header-thumbnail.jpg',
				'description' => 'インテリア'
			),
			'breads' => array(
				'url' => '%s/img/custom-header02.jpg',
				'thumbnail_url' => '%s/img/custom-header02-thumbnail.jpg',
				'description' => 'パン'
			),
			'kitchen' => array(
				'url' => '%s/img/custom-header03.jpg',
				'thumbnail_url' => '%s/img/custom-header03-thumbnail.jpg',
				'description' => 'キッチン'
			),
			'bottles' => array(
				'url' => '%s/img/custom-header04.jpg',
				'thumbnail_url' => '%s/img/custom-header04-thumbnail.jpg',
				'description' => 'ボトル'
			),
		)
	);

	// 管理画面でなければ、initフックでadd_jqueryが実行されるようアクションフックの登録を行います。
	// これは、Webサイト表示時でもjQueryを利用するためです。
	if ( ! is_admin() ) {
		add_action( 'init', 'add_jquery' );
	}

	// 抜粋文の最後に付与される３点リーダーの文字列をCafe Miraggioテーマ用のもの変更します。
	function miraggio_excerpt_more() {
		return ' ...';
	}
	add_filter( 'excerpt_more', 'miraggio_excerpt_more' );
	
	/**
	 * 抜粋文の文字数をCafe Miraggioテーマ用に180文字に変更します。
	 * ただし、正しく動作させるためにはMultibyte Patchの有効化が必要です。
	 */
	function miraggio_excerpt_length() {
		return 180;
	}
	add_filter( 'excerpt_length', 'miraggio_excerpt_length' );
	add_filter( 'excerpt_mblength', 'miraggio_excerpt_length' );
}
endif;

// jQueryをロードするようscriptキューに対して登録を行います。
function add_jquery() {
	wp_enqueue_script('jquery');
}


/**
 * ナビゲーションが登録されていない場合に表示されるメニューにトップページへのメニューが含まれるようにします。
 */
function miraggio_page_menu_args( $args ) {
	$args['show_home'] = true;
	return $args;
}
add_filter( 'wp_page_menu_args', 'miraggio_page_menu_args' );

/**
 * miraggio_setup 同様にmiraggio_commentも子テーマのfunctions.phpで定義すればオーバーライドが可能です。
 * 子テーマでコメントのソースを調整したい場合は、子テーマのfunctions.phpでmiraggio_comment関数を定義すれば、そちらが優先され、このmiraggio_comment関数は読み込まれません。
 * コメントを表示するhtmlを変更したい場合に利用してください。
 */
if ( ! function_exists( 'miraggio_comment' ) ) :
/**
 * コメントとトラックバック・ピンバック表示のテンプレートです。
 */
function miraggio_comment( $comment, $args, $depth ) {
	$GLOBALS['comment'] = $comment;
	switch ( $comment->comment_type ) :
		case '' :
	?>
	<li <?php comment_class(); ?> id="li-comment-<?php comment_ID(); ?>">
		<div id="comment-<?php comment_ID(); ?>">
		<div class="comment-author vcard">
			<?php echo get_avatar( $comment, 40 ); ?>
			<?php printf( '<cite class="fn">%s</cite>', get_comment_author_link() ); ?>
		</div><!-- .comment-author .vcard -->
		<?php if ( $comment->comment_approved == '0' ) : ?>
			<em class="comment-awaiting-moderation"><?php echo 'このコメントは管理者の承認待ちです。'; ?></em>
			<br />
		<?php endif; ?>

		<div class="comment-meta commentmetadata"><a href="<?php echo esc_url( get_comment_link( $comment->comment_ID ) ); ?>">
			<?php
				echo get_comment_date() . ' at ' . get_comment_time(); ?></a><?php edit_comment_link( '編集', ' ' );
			?>
		</div><!-- .comment-meta .commentmetadata -->

		<div class="comment-body"><?php comment_text(); ?></div>

		<div class="reply">
			<?php comment_reply_link( array_merge( $args, array( 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
		</div><!-- .reply -->
	</div><!-- #comment-##  -->

	<?php
			break;
		case 'pingback'  :
		case 'trackback' :
	?>
	<li class="post pingback">
		<p>ピンバック： <?php comment_author_link(); ?><?php edit_comment_link( '編集', ' ' ); ?></p>
	<?php
			break;
	endswitch;
}
endif;

/**
 * ウィジェットエリアの登録を行います。
 * 子テーマでエリアの設定を変更したい場合は、after_setup_themeにおいてmiraggio_widgets_initをremove_filterで
 * フックを削除し、別途ウィジェットの登録を行う必要があります。
 */
function miraggio_widgets_init() {
	/*
	 * register_sidebar関数は、ウィジェットエリアの登録を行う関数です。
	 * パラメーターは、配列で指定し、それぞれの役割は次の通りです。
	 * name				: ウィジェット管理画面に表示されるウィジェットエリア名
	 * id				: エリアのID（スラッグ）
	 * description		: 管理画面に表示されるウィジェットエリアの説明文
	 * before_widget	: ウィジェットエリア開始時に出力するコード（エリアのラッパー要素の始点）
	 * after_widget		: ウィジェットエリア終了時に出力するコード（エリアのラッパー要素の終点）
	 * before_title		: ウィジェットのタイトルを表示する前に出力するコード
	 * after_title		: ウィジェットのタイトルを表示した後に出力するコード
	*/ 
	
	register_sidebar( array(
		'name' => 'サイドバーウィジェットエリア（上）',
		'id' => 'primary-widget-area',
		'description' => 'サイドバー上部のウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'サイドバーウィジェットエリア（下）',
		'id' => 'secondary-widget-area',
		'description' => 'サイドバー下部のウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'トップページコンテンツウィジェット１',
		'id' => 'home-content-widget-upper',
		'description' => 'トップページ上段のウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'トップページコンテンツウィジェット２',
		'id' => 'home-content-widget-lowwer',
		'description' => 'トップページ下段のウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'ヘッダーウィジェットエリア',
		'id' => 'header-widget-area',
		'description' => 'ヘッダーのウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'フッターウィジェットエリア１',
		'id' => 'first-footer-widget-area',
		'description' => 'フッターのウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'フッターウィジェットエリア２',
		'id' => 'second-footer-widget-area',
		'description' => 'フッターのウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'フッターウィジェットエリア３',
		'id' => 'third-footer-widget-area',
		'description' => 'フッターのウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );

	register_sidebar( array(
		'name' => 'フッターウィジェットエリア４',
		'id' => 'fourth-footer-widget-area',
		'description' => 'フッターのウィジェットエリア',
		'before_widget' => '<aside id="%1$s" class="widget-container %2$s">',
		'after_widget' => '</aside>',
		'before_title' => '<h1 class="widget-title">',
		'after_title' => '</h1>',
	) );
}
/**
 * widget_initフックでウィジェットエリアの登録を行うmiraggio_widgets_initが実行されるようにします。
 * widget_initは、「WordPressの実行フローを視覚化してみる」ではwp-settings.phpの7のinitフックの最初
 * （優先度1）で行われます。
 */
add_action( 'widgets_init', 'miraggio_widgets_init' );


// サイトタイトル画像の幅と高さを定義します。
define( 'SITE_TITLE_IMAGE_WIDTH', 300 );
define( 'SITE_TITLE_IMAGE_HEIGHT', 80 );

/**
 * site_title_image_formは、管理画面の[外観]→[ヘッダー]において、サイトタイトル画像のアップロードフォームを
 * 出力します。
 */
function site_title_image_form() {
	if ( $site_title_image_src = get_theme_mod( 'site_title_image' ) ) {
		$title_preview = '<div style="width: ' . SITE_TITLE_IMAGE_WIDTH . 'px; height: ' . SITE_TITLE_IMAGE_HEIGHT . 'px; background: url( ' . $site_title_image_src . ' ) no-repeat;">&nbsp;</div>';
	} else {
		$title_preview = '<p>サイトタイトル画像が設定されていません。</p>';
	}
?>
<?php wp_nonce_field( 'custom-header-options', '_wpnonce-custom-header-options' ); ?>

<?php submit_button( null, 'primary', 'save-header-options' ); ?>
</form>
<hr style="border: none; border-bottom: solid 1px #dfdfdf;" />
<form enctype="multipart/form-data" method="post" action="<?php echo esc_attr( add_query_arg( 'step', 1 ) ) ?>">
<h3>サイトタイトル画像</h3>
<table class="form-table">
	<tbody>
		<tr>
			<th>プレビュー</th>
			<td><?php echo $title_preview; ?></td>
		</tr>
		<tr>
			<th>画像をアップロード</th>
			<td>
				<input type="file" sise="50" id="site_title" name="site_title" />
			</td>
		</tr>
		<tr>
			<th>画像をリセット</th>
			<td>
				<p>この操作を行うとサイトタイトル画像を削除します。変更した設定は復元できません。</p>
				<input type="submit" name="remove_site_title_image" id="remove_site_title_image" class="button" value="サイトタイトル画像を削除"  />
			</td>
		</tr>
	</tbody>
</table>
<?php
}
/**
 * カスタムヘッダー管理画面にあるcustom_header_optionsに対して、site_title_image_formのフックを追加します。
 * パラメーターの1000は優先度で数字が大きくなるほど、実行タイミングが遅くなります。1000と指定したのは、なるべく
 * 実行を遅く（下に表示）するための指定です。
 */
add_action( 'custom_header_options', 'site_title_image_form', 1000 );

/**
 * site_title_image_upload_processでは、カスタムヘッダー管理画面に追加したサイトタイトル画像で指定したファイルの
 * アップロードとテーマ設定の登録処理を行います。
 */
function site_title_image_upload_process() {
	if ( isset( $_POST['remove_site_title_image'] ) ) {
		check_admin_referer( 'custom-header-options', '_wpnonce-custom-header-options' );
		remove_theme_mod( 'site_title_image' );
		return;
	} elseif ( isset( $_FILES['site_title'] ) ) {
		check_admin_referer( 'custom-header-options', '_wpnonce-custom-header-options');

		if ( ! current_theme_supports( 'custom-header-uploads' ) )
			wp_die( 'Cheatin&#8217; uh?' );

		$overrides = array('test_form' => false);
		$file = wp_handle_upload($_FILES['site_title'], $overrides );

		if ( isset($file['error'] ) )
			wp_die( $file['error'],  __( 'Image Upload Error' ) );

		$url = $file['url'];
		$type = $file['type'];
		$file = $file['file'];
		$filename = basename($file);

		// Construct the object array
		$object = array(
			'post_title' => $filename,
			'post_content' => $url,
			'post_mime_type' => $type,
			'guid' => $url
		);

		$id = wp_insert_attachment($object, $file);
		list( $width, $height, $type, $attr ) = getimagesize( $file );

		wp_update_attachment_metadata( $id, wp_generate_attachment_metadata( $id, $file ) );

		set_theme_mod( 'site_title_image', esc_url($url) );
	}
}

/**
 * 
 * アップロードとテーマ設定の登録処理を行います。
 */
function add_upload_process_action() {
	global $custom_image_header;
	add_action( 'admin_head-' . $custom_image_header->page, 'site_title_image_upload_process' );
}
add_action( 'admin_menu', 'add_upload_process_action', 11 );

/*
 * パンくずナビのカスタムテンプレートタグです。
 * function_existsでの分岐により、子テーマでのオーバーライドを可能にし、プラグインとの関数競合を防ぎます。
 * 本カスタムテンプレートタグは、プラグインの Prime Strategy Bread Crumb ( http://wordpress.org/extend/plugins/prime-strategy-bread-crumb/ )と
 * ほぼ同じ仕様ですが、プラグインが書籍リリース以後にアップデートしている場合は、プラグインの利用をおすすめします。
 * パラメータでカスタマイズしやすいWordPressのパンくずナビを作ってみた（ http://www.warna.info/archives/757/ ）
 */
if ( ! function_exists( 'bread_crumb' ) ) :

function bread_crumb( $args = '' ) {
	$default = array(
		'type'				=> 'list',
		'home_label'		=> 'トップページ',
		'search_label'		=> '『%s』の検索結果',
		'404_label'			=> '404 Not Found',
		'category_label'	=> '%s',
		'tag_label'			=> '%s',
		'taxonomy_label'	=> '%s',
		'author_label'		=> '%s',
		'attachment_label'	=> '%s',
		'year_label'		=> '%s年',
		'month_label'		=> '%s月',
		'day_label'			=> '%s日',
		'post_type_label'	=> '%s',
		'joint_string'		=> ' &gt; ',
		'navi_element'		=> '',
		'elm_class'			=> 'bread_crumb',
		'elm_id'			=> '',
		'li_class'			=> '',
		'class_prefix'		=> '',
		'current_class'		=> 'current',
		'indent'			=> 0,
		'echo'				=> true,
	);
	$default = apply_filters( 'bread_crumb_default', $default );
	$args = wp_parse_args( $args, $default );

	$elm = in_array( $args['navi_element'], array( 'nav', 'div', '' ) ) ? $args['navi_element'] : 'div';
	$args['elm_id'] = is_array( $args['elm_id'] ) ? $default['elm_id'] : $args['elm_id'];
	$args['elm_id'] = preg_replace( '/[^\w_-]+/', '', $args['elm_id'] );
	$args['elm_id'] = preg_replace( '/^[\d_-]+/', '', $args['elm_id'] );

	$args['class_prefix'] = is_array( $args['class_prefix'] ) ? $default['class_prefix'] : $args['class_prefix'];
	$args['class_prefix'] = preg_replace( '/[^\w_-]+/', '', $args['class_prefix'] );
	$args['class_prefix'] = preg_replace( '/^[\d_-]+/', '', $args['class_prefix'] );

	$args['elm_class'] = sanitize_attr_classes( $args['elm_class'], $args['class_prefix'] );
	$args['li_class'] = sanitize_attr_classes( $args['li_class'], $args['class_prefix'] );
	$args['current_class'] = sanitize_attr_classes( $args['current_class'], $args['class_prefix'] );
	$args['current_class'] = $args['current_class'] ? $args['current_class'] : $args['class_prefix'] . $default['current_class'];
	$args['echo'] = uniform_boolean( $args['echo'], $default['echo'] );

	$tabs = str_repeat( "\t", (int)$args['indent'] );

	$bread_crumb_arr = get_bread_crumb_array( $args );

	$elm_attrs = '';
	if ( $args['elm_id'] ) {
		$elm_attrs = ' id="' . $args['elm_id'] . '"';
	}
	if ( $args['elm_class'] ) {
		$elm_attrs .= ' class="' . $args['elm_class'] . '"';
	}

	$output = '';
	$elm_tabs = '';

	$output = '';
	if ( $elm ) {
		$elm_tabs = "\t";
		$output = $tabs . '<' . $elm;
		if ( $elm_attrs ) {
			$output .= $elm_attrs . ">\n";
		}
	}

	if ( $args['type'] == 'string' ) {
		$output = array();
		$cnt = 1;
		foreach ( $bread_crumb_arr as $ancestor ) {
			if ( $cnt == count( $bread_crumb_arr ) ) {
				$output[] = '<strong class="' . $args['current_class'] . '">' . apply_filters( 'the_title', $ancestor['title'] ) . '</strong>';
			} else {
				$output[] = '<a href="' . $ancestor['link'] . '">' . apply_filters( 'the_title', $ancestor['title'] ) . '</a>';
			}
			$cnt++;
		}
		$output = $elm_tabs . $tabs . implode( esc_html( $args['joint_string'] ), $output );
		$output = apply_filters( 'bread_crumb_after', $output, $args );

	} else {
		$output .= $elm_tabs . $tabs . '<ul';
		if ( ! $elm && $elm_attrs ) {
			$output .= $elm_attrs;
		}
		$output .= ">\n";

		$output = apply_filters( 'bread_crumb_before', $output, $args );

		$cnt = 1;
		foreach ( $bread_crumb_arr as $ancestor ) {
			$classes = array();
			$classes[] = $args['class_prefix'] . 'level-' . $cnt;
			if ( $cnt == 1 ) {
				$classes[] = $args['class_prefix'] . 'top';
			} else {
				$classes[] = $args['class_prefix'] . 'sub';
			}
			if ( $cnt == count( $bread_crumb_arr ) ) {
				$classes[] = $args['class_prefix'] . 'tail';
				$output .= $elm_tabs . $tabs . '	<li class="' . implode( ' ', $classes );
				if ( $args['li_class'] ) {
					$output .= ' ' . $args['li_class'];
				}
				$output .= ' ' .  $args['current_class'];
				$output .= '">' . apply_filters( 'the_title', $ancestor['title'] ) . '</li>' . "\n";
			} else {
				$output .= $elm_tabs . $tabs . '	<li class="' . implode( ' ', $classes );
				if ( $args['li_class'] ) {
					$output .= ' ' . $args['li_class'];
				}
				$output .= '"><a href="' . $ancestor['link'] . '">' . apply_filters( 'the_title', $ancestor['title'] ) . '</a></li>' . "\n";
			}
			$cnt++;
		}
		$output = apply_filters( 'bread_crumb_after', $output, $args );

		$output .= $elm_tabs . $tabs . '</ul>' . "\n";
	}

	if ( $elm ) {
		$output .= $tabs . '</' . $elm . ">\n";
	}

	$output = apply_filters( 'bread_crumb', $output, $args );

	if ( $args['echo'] ) {
		echo $output;
	} else {
		return $output;
	}
}
endif;

/*
 * パンくずナビを表示するための配列データを生成します。
 * カスタマイズしたサイト構造にパンくずナビを合わせる場合、bread_crumb_arrayにフックして配列データの調整を行えば、
 * サイト構造に合わせたパンくずナビの表示を実現することが可能です。
 */ 
function get_bread_crumb_array( $args ) {
	global $post;

	$bread_crumb_arr = array();
	$bread_crumb_arr[] = array( 'title' => $args['home_label'], 'link' => get_bloginfo( 'url' ) . '/' );
	$bread_crumb_arr = add_posts_page_array( $bread_crumb_arr );
	if ( is_404() ) {
		$bread_crumb_arr[] = array( 'title' => $args['404_label'], 'link' => false );
	} elseif ( is_search() ) {
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['search_label'], get_search_query() ), 'link' => false );
	} elseif ( is_tax() ) {
		$taxonomy = get_query_var( 'taxonomy' );
		$term = get_term_by( 'slug', get_query_var( 'term' ), $taxonomy );
		if ( is_taxonomy_hierarchical( $taxonomy ) && $term->parent != 0 ) {
			$ancestors = array_reverse( get_ancestors( $term->term_id, $taxonomy ) );
			foreach ( $ancestors as $ancestor_id ) {
				$ancestor = get_term( $ancestor_id, $taxonomy );
				$bread_crumb_arr[] = array( 'title' => $ancestor->name, 'link' => get_term_link( (int)$ancestor, $term->slug ) );
			}
		}
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['taxonomy_label'], $term->name ), 'link' => get_term_link( (int)$term->term_id, $term->slug ) );
	} elseif ( is_attachment() ) {
		if ( $post->post_parent ) {
			if ( $parent_post = get_post( $post->post_parent ) ) {
				$singular_bread_crumb_arr = get_singular_bread_crumb_array( $parent_post, $args );
				$bread_crumb_arr = array_merge( $bread_crumb_arr, $singular_bread_crumb_arr );
				$bread_crumb_arr[] = array( 'title' => $parent_post->post_title, 'link' => get_permalink( $parent_post->ID ) );
			}
		}
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['attachment_label'], $post->post_title ), 'link' => get_permalink( $post->ID ) );
	} elseif ( is_singular() && ! is_front_page() ) {
		$singular_bread_crumb_arr = get_singular_bread_crumb_array( $post, $args );
		$bread_crumb_arr = array_merge( $bread_crumb_arr, $singular_bread_crumb_arr );
		$bread_crumb_arr[] = array( 'title' => $post->post_title, 'link' => get_permalink( $post->ID ) );
	} elseif ( is_category() ) {
		global $cat;

		$category = get_category( $cat );
		if ( $category->parent != 0 ) {
			$ancestors = array_reverse( get_ancestors( $category->term_id, 'category' ) );
			foreach ( $ancestors as $ancestor_id ) {
				$ancestor = get_category( $ancestor_id );
				$bread_crumb_arr[] = array( 'title' => $ancestor->name, 'link' => get_category_link( $ancestor->term_id ) );
			}
		}
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['category_label'], $category->name ), 'link' => get_category_link( $cat ) );
	} elseif ( is_tag() ) {
		global $tag_id;
		$tag = get_tag( $tag_id );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['tag_label'], $tag->name ), 'link' => get_tag_link( $tag_id ) );
	} elseif ( is_author() ) {
		$author = get_query_var( 'author' );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['author_label'], get_the_author_meta( 'display_name', get_query_var( 'author' ) ) ), 'link' => get_author_posts_url( $author ) );
	} elseif ( is_day() ) {
		$year = get_query_var( 'year' );
		$month = get_query_var( 'monthnum' );
		$day = get_query_var( 'day' );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['year_label'], $year ), 'link' => get_year_link( $year ) );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['month_label'], $month ), 'link' => get_month_link( $year, $month ) );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['day_label'], $day ), 'link' => get_day_link( $year, $month, $day ) );
	} elseif ( is_month() ) {
		if ( $m = get_query_var( 'm' ) ) {
			$year = substr( $m, 0, 4 );
			$month = substr( $m, -1, 2 );
		} else {
			$year = get_query_var( 'year' );
			$month = get_query_var( 'monthnum' );
		}
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['year_label'], $year ), 'link' => get_year_link( $year ) );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['month_label'], $month ), 'link' => get_month_link( $year, $month ) );
	} elseif ( is_year() ) {
		if ( $m = get_query_var( 'm' ) ) {
			$year = substr( $m, 0, 4 );
		} else {
			$year = get_query_var( 'year' );
		}
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['year_label'], $year ), 'link' => get_year_link( $year ) );
	} elseif ( is_post_type_archive() ) {
		$post_type = get_post_type_object( get_query_var( 'post_type' ) );
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['post_type_label'], $post_type->label ), 'link' => get_post_type_archive_link( $post_type->name ) );
	}
	return apply_filters( 'bread_crumb_array', $bread_crumb_arr, $args );
}


// 投稿・固定ページ・カスタム投稿表示時のパンくずナビデータを生成します。
function get_singular_bread_crumb_array( $post, $args ) {
	$bread_crumb_arr = array();
	$post_type = get_post_type_object( $post->post_type );

	if ( $post_type && $post_type->has_archive ) {
		$bread_crumb_arr[] = array( 'title' => sprintf( $args['post_type_label'], $post_type->label ), 'link' => get_post_type_archive_link( $post_type->name ) );
	}

	if ( is_post_type_hierarchical( $post_type->name ) ) {
		$ancestors = array_reverse( get_post_ancestors( $post ) );
		if ( count( $ancestors ) ) {
			$ancestor_posts = get_posts( 'post_type=' . $post_type->name . '&include=' . implode( ',', $ancestors ) );
			foreach( $ancestors as $ancestor ) {
				foreach ( $ancestor_posts as $ancestor_post ) {
					if ( $ancestor == $ancestor_post->ID ) {
						$bread_crumb_arr[] = array( 'title' => $ancestor_post->post_title, 'link' => get_permalink( $ancestor_post->ID ) );
					}
				}
			}
		}
	} else {
		$post_type_taxonomies = get_object_taxonomies( $post_type->name, false );
		if ( is_array( $post_type_taxonomies ) && count( $post_type_taxonomies ) ) {
			foreach( $post_type_taxonomies as $tax_slug => $taxonomy ) {
				if ( $taxonomy->hierarchical ) {
					$terms = get_the_terms( $post->ID, $tax_slug );
					if ( $terms ) {
						$term = array_shift( $terms );
						if ( $term->parent != 0 ) {
							$ancestors = array_reverse( get_ancestors( $term->term_id, $tax_slug ) );
							foreach ( $ancestors as $ancestor_id ) {
								$ancestor = get_term( $ancestor_id, $tax_slug );
								$bread_crumb_arr[] = array( 'title' => $ancestor->name, 'link' => get_term_link( $ancestor, $tax_slug ) );
							}
						}
						$bread_crumb_arr[] = array( 'title' => $term->name, 'link' => get_term_link( $term, $tax_slug ) );
						break;
					}
				}
			}
		}
	}
	return $bread_crumb_arr;
}


// 投稿ページのパンくずナビデータを追加します。
function add_posts_page_array( $bread_crumb_arr ) {
	if ( is_page() || is_front_page() ) {
		return $bread_crumb_arr;
	} elseif ( is_category() ) {
		$tax = get_taxonomy( 'category' );
		if ( count( $tax->object_type ) != 1 || $tax->object_type[0] != 'post' ) {
			return $bread_crumb_arr;
		}
	} elseif ( is_tag() ) {
		$tax = get_taxonomy( 'post_tag' );
		if ( count( $tax->object_type ) != 1 || $tax->object_type[0] != 'post' ) {
			return $bread_crumb_arr;
		}
	} elseif ( is_tax() ) {
		$tax = get_taxonomy( get_query_var( 'taxonomy' ) );
		if ( count( $tax->object_type ) != 1 || $tax->object_type[0] != 'post' ) {
			return $bread_crumb_arr;
		}
	} elseif ( is_home() && ! get_query_var( 'pagename' ) ) {
		return $bread_crumb_arr;
	} else {
		$post_type = get_query_var( 'post_type' ) ? get_query_var( 'post_type' ) : 'post';
		if ( $post_type != 'post' ) {
			return $bread_crumb_arr;
		}
	}
	if ( get_option( 'show_on_front' ) == 'page' && $posts_page_id = get_option( 'page_for_posts' ) ) {
		$posts_page = get_post( $posts_page_id );
		$bread_crumb_arr[] = array( 'title' => $posts_page->post_title, 'link' => get_permalink( $posts_page->ID ) );
	}
	return $bread_crumb_arr;
}

/*
 * ページナビを出力するカスタムテンプレートタグを定義します。
 * function_existsでの分岐により、子テーマでのオーバーライドを可能にし、プラグインとの関数競合を防ぎます。
 * 本カスタムテンプレートタグは、プラグインの Prime Strategy Page Navi ( http://wordpress.org/extend/plugins/prime-strategy-page-navi/ )と
 * ほぼ同じ仕様ですが、プラグインが書籍リリース以後にアップデートしている場合は、プラグインの利用をおすすめします。
 * 利用できるパラメーターは、下記を参照してください。
 * パラメータでカスタマイズしやすいWordPressのページナビを作ってみた（ http://www.warna.info/archives/723/ ）
 */ 

if ( ! function_exists( 'page_navi' ) ) :
// ページナビを出力するテンプレートタグを定義します。
function page_navi( $args = '' ) {
	global $wp_query;

	if ( ! ( is_archive() || is_home() || is_search() ) ) { return; }
	if ( ! $wp_query->max_num_pages ) { return; }
	$default = array(
		'items'			 => 11,
		'edge_type'			=> 'none',
		'show_adjacent'	 => true,
		'prev_label'		=> '&lt;',
		'next_label'		=> '&gt;',
		'show_boundary'	 => true,
		'first_label'	   => '&laquo;',
		'last_label'		=> '&raquo;',
		'show_num'		  => false,
		'num_position'	  => 'before',
		'num_format'		=> '<span>%d/%d</span>',
		'echo'			  => true,
		'navi_element'	  => '',
		'elm_class'		 => 'page_navi',
		'elm_id'			=> '',
		'li_class'		  => '',
		'current_class'	 => 'current',
		'current_format'	=> '<span>%d</span>',
		'class_prefix'	  => '',
		'indent'			=> 0
	);
	$default = apply_filters( 'page_navi_default', $default );

	$args = wp_parse_args( $args, $default );

	$max_page_num = $wp_query->max_num_pages;
	$current_page_num = get_query_var( 'paged' ) ? get_query_var( 'paged' ) : 1;

	$elm = in_array( $args['navi_element'], array( 'nav', 'div', '' ) ) ? $args['navi_element'] : 'div';

	$args['items'] = absint( $args['items'] ) ? absint( $args['items'] ) : $default['items'];
	$args['elm_id'] = is_array( $args['elm_id'] ) ? $default['elm_id'] : $args['elm_id'];
	$args['elm_id'] = preg_replace( '/[^\w_-]+/', '', $args['elm_id'] );
	$args['elm_id'] = preg_replace( '/^[\d_-]+/', '', $args['elm_id'] );

	$args['class_prefix'] = is_array( $args['class_prefix'] ) ? $default['class_prefix'] : $args['class_prefix'];
	$args['class_prefix'] = preg_replace( '/[^\w_-]+/', '', $args['class_prefix'] );
	$args['class_prefix'] = preg_replace( '/^[\d_-]+/', '', $args['class_prefix'] );

	$args['elm_class'] = sanitize_attr_classes( $args['elm_class'], $args['class_prefix'] );
	$args['li_class'] = sanitize_attr_classes( $args['li_class'], $args['class_prefix'] );
	$args['current_class'] = sanitize_attr_classes( $args['current_class'], $args['class_prefix'] );
	$args['current_class'] = $args['current_class'] ? $args['current_class'] : $default['current_class'];

	$args['show_adjacent'] = uniform_boolean( $args['show_adjacent'], $default['show_adjacent'] );
	$args['show_boundary'] = uniform_boolean( $args['show_boundary'], $default['show_boundary'] );
	$args['show_num'] = uniform_boolean( $args['show_num'], $default['show_num'] );
	$args['echo'] = uniform_boolean( $args['echo'], $default['echo'] );

	$tabs = str_repeat( "\t", (int)$args['indent'] );
	$elm_tabs = '';

	$befores = $current_page_num - floor( ( $args['items'] - 1 ) / 2 );
	$afters = $current_page_num + ceil( ( $args['items'] - 1 ) / 2 );

	if ( $max_page_num <= $args['items'] ) {
		$start = 1;
		$end = $max_page_num;
	} elseif ( $befores <= 1 ) {
		$start = 1;
		$end = $args['items'];
	} elseif ( $afters >= $max_page_num ) {
		$start = $max_page_num - $args['items'] + 1;
		$end = $max_page_num;
	} else {
		$start = $befores;
		$end = $afters;
	}

	$elm_attrs = '';
	if ( $args['elm_id'] ) {
		$elm_attrs = ' id="' . $args['elm_id'] . '"';
	}
	if ( $args['elm_class'] ) {
		$elm_attrs .= ' class="' . $args['elm_class'] . '"';
	}

	$num_list_item = '';
	if ( $args['show_num'] ) {
		$num_list_item = '<li class="page_nums';
		if ( $args['li_class'] ) {
			$num_list_item .= ' ' . $args['li_class'];
		}
		$num_list_item .= '">' . sprintf( $args['num_format'], $current_page_num, $max_page_num ) . "</li>\n";
	}

	$page_navi = '';
	if ( $elm ) {
		$elm_tabs = "\t";
		$page_navi = $tabs . '<' . $elm;
		if ( $elm_attrs ) {
			$page_navi .= $elm_attrs . ">\n";
		}
	}

	$page_navi .= $elm_tabs . $tabs . '<ul';
	if ( ! $elm && $elm_attrs ) {
		$page_navi .= $elm_attrs;
	}
	$page_navi .= ">\n";

	if ($args['num_position'] != 'after' && $num_list_item ) {
		$page_navi .= "\t" . $elm_tabs . $tabs . $num_list_item;
	}
	if ( $args['show_boundary'] && ( $current_page_num != 1 || in_array( $args['edge_type'], array( 'span', 'link' ) ) ) ) {
		$page_navi .= "\t" . $elm_tabs . $tabs . '<li class="' . $args['class_prefix'] . 'first';
		if ( $args['li_class'] ) {
			$page_navi .= ' ' . $args['li_class'];
		}
		if ( $args['edge_type'] == 'span' && $current_page_num == 1 ) {
			$page_navi .= '"><span>' . esc_html( $args['first_label'] ) . '</span></li>' . "\n";
		} else {
			$page_navi .= '"><a href="' . get_pagenum_link() . '">' . esc_html( $args['first_label'] ) . '</a></li>' . "\n";
		}
	}

	if ( $args['show_adjacent'] && ( $current_page_num != 1 || in_array( $args['edge_type'], array( 'span', 'link' ) ) ) ) {
		$previous_num = max( 1, $current_page_num - 1 );
		$page_navi .= "\t" . $elm_tabs . $tabs . '<li class="' . $args['class_prefix'] . 'previous';
		if ( $args['li_class'] ) {
			$page_navi .= ' ' . $args['li_class'];
		}
		if ( $args['edge_type'] == 'span' && $current_page_num == 1 ) {
			$page_navi .= '"><span>' . esc_html( $args['prev_label'] ) . '</span></li>' . "\n";
		} else {
			$page_navi .= '"><a href="' . get_pagenum_link( $previous_num ) . '">' . esc_html( $args['prev_label'] ) . '</a></li>' . "\n";
		}
	}

	for ( $i = $start; $i <= $end; $i++ ) {
		$page_navi .= "\t" . $elm_tabs . $tabs . '<li class="';
		if ( $i == $current_page_num ) {
			$page_navi .= $args['current_class'];
			if ( $args['li_class'] ) {
				$page_navi .= ' ' . $args['li_class'];
			}
			$page_navi .= '">' . sprintf( $args['current_format'], $i ) . "</li>\n";
		} else {
			$delta = absint( $i - $current_page_num );
			$b_f = $i < $current_page_num ? 'before' : 'after';
			$page_navi .= $args['class_prefix'] . $b_f . ' ' . $args['class_prefix'] . 'delta-' . $delta;
			if ( $i == $start ) {
				$page_navi .= ' ' . $args['class_prefix'] . 'head';
			} elseif ( $i == $end ) {
				$page_navi .= ' ' . $args['class_prefix'] . 'tail';
			}
			if ( $args['li_class'] ) {
				$page_navi .= ' ' . $args['li_class'] . '"';
			}
			$page_navi .= '"><a href="' . get_pagenum_link( $i ) . '">' . $i . "</a></li>\n";
		}
	}

	if ( $args['show_adjacent'] && ( $current_page_num != $max_page_num || in_array( $args['edge_type'], array( 'span', 'link' ) ) ) ) {
		$next_num = min( $max_page_num, $current_page_num + 1 );
		$page_navi .= "\t" . $elm_tabs . $tabs . '<li class="' . $args['class_prefix'] . 'next';
		if ( $args['li_class'] ) {
			$page_navi .= ' ' . $args['li_class'];
		}
		if ( $args['edge_type'] == 'span' && $current_page_num == $max_page_num ) {
			$page_navi .= '"><span>' . esc_html( $args['next_label'] ) . '</span></li>' . "\n";
		} else {
			$page_navi .= '"><a href="' . get_pagenum_link( $next_num ) . '">' . esc_html( $args['next_label'] ) . '</a></li>' . "\n";

		}
	}

	if ( $args['show_boundary'] && ( $current_page_num != $max_page_num || in_array( $args['edge_type'], array( 'span', 'link' ) ) ) ) {
		$page_navi .= "\t" . $elm_tabs . $tabs . '<li class="' . $args['class_prefix'] . 'last';
		if ( $args['li_class'] ) {
			$page_navi .= ' ' . $args['li_class'];
		}
		if ( $args['edge_type'] == 'span' && $current_page_num == $max_page_num ) {
			$page_navi .= '"><span>' . esc_html( $args['last_label'] ) . '</span></li>' . "\n";
		} else {
			$page_navi .= '"><a href="' . get_pagenum_link( $max_page_num ) . '">' . esc_html( $args['last_label'] ) . '</a></li>' . "\n";
		}
	}

	if ($args['num_position'] == 'after' && $num_list_item ) {
		$page_navi .= "\t" . $elm_tabs . $tabs . $num_list_item;
	}

	$page_navi .= $elm_tabs . $tabs . "</ul>\n";

	if ( $elm ) {
		$page_navi .= $tabs . '</' . $elm . ">\n";
	}

	$page_navi = apply_filters( 'page_navi', $page_navi );

	if ( $args['echo'] ) {
		echo $page_navi;
	} else {
		return $page_navi;
	}
}
endif;


// id属性やclass属性として出力される文字列の正規化を行います。
// パンくずナビおよびページナビ機能で利用しています。
function sanitize_attr_classes( $classes, $prefix = '' ) {
	if ( ! is_array( $classes ) ) {
		$classes = preg_replace( '/[^\s\w_-]+/', '', $classes );
		$classes = preg_split( '/[\s]+/', $classes );
	}

	foreach ( $classes as $key => $class ) {
		if ( is_array( $class ) ) {
			unset( $classes[$key] );
		} else {
			$class = preg_replace( '/[^\w_-]+/', '', $class );
			$class = preg_replace( '/^[\d_-]+/', '', $class );
			if ( $class ) {
				$classes[$key] = $prefix . $class;
			}
		}
	}
	$classes = implode( ' ', $classes );

	return $classes;
}


// 真偽値をとるパラメータの正規化を行います。
// パンくずナビおよびページナビ機能で利用しています。
function uniform_boolean( $arg, $default = true ) {
	if ( is_numeric( $arg ) ) {
		$arg = (int)$arg;
	}
	if ( is_string( $arg ) ) {
		$arg = strtolower( $arg );
		if ( $arg == 'false' ) {
			$arg = false;
		} elseif ( $arg == 'true' ) {
			$arg = true;
		} else {
			$arg = $default;
		}
	}
	return $arg;
}


/*
 * ウィジェット内に登録されているウィジェットの数を計算し、登録数に応じたclass名を表示するカスタムテンプレートタグのwp_widget_num_classを定義します。
 * パラメーターは下記の通り
 * $index  : 登録数をカウントするウィジェットエリアのスラッグを文字列として指定する。register_sidebarで指定したidの値。
 * $prefix : 表示するclass名の接頭辞。初期値は、'widgets-'
 * $suffix : 表示するclass名の接尾辞。初期値は、なし
 * $echo   : class名を出力するか、文字列として取得するか。初期値はtrue（出力する）
 */
function wp_widget_num_class( $index = 1, $prefix = 'widgets-', $suffix = '', $echo = true ) {
	global $wp_registered_sidebars;

	if ( is_int($index) ) {
		$index = "sidebar-$index";
	} else {
		$index = sanitize_title($index);
		foreach ( (array) $wp_registered_sidebars as $key => $value ) {
			if ( sanitize_title($value['name']) == $index ) {
				$index = $key;
				break;
			}
		}
	}

	$sidebars_widgets = wp_get_sidebars_widgets();

	if ( empty($wp_registered_sidebars[$index]) || !array_key_exists($index, $sidebars_widgets) || !is_array($sidebars_widgets[$index]) || empty($sidebars_widgets[$index]) )
		return false;

	$widget_num_class = $prefix . count( $sidebars_widgets[$index] ) . $suffix;

	if ( $echo ) {
		echo $widget_num_class;
	} else {
		return $widget_num_class;
	}
}

/*
 * パラメーターで指定された複数のウィジェットエリアの中で、ウィジェットの表示があるウィジェット数に応じたclass名を出力する
 * カスタムテンプレートタグの wp_area_widget_num_class を定義します。
 * パラメーターは下記の通り
 * $widgets : 登録数をカウントするウィジェットエリアのスラッグを配列で指定する。register_sidebarで指定したidの値。
 * $prefix  : 表示するclass名の接頭辞。初期値は、'area-widgets-'
 * $suffix  : 表示するclass名の接尾辞。初期値は、なし
 * $echo    : class名を出力するか、文字列として取得するか。初期値はtrue（出力する）
 */
function wp_area_widget_num_class( $widgets, $prefix = 'area-widgets-', $suffix = '', $echo = true ) {
	
	if ( ! is_array( $widgets ) || ! count( $widgets ) ) {
		return false;
	}

	$widgets_in_area = 0;
	foreach ( $widgets as $index ) {
		if ( is_active_sidebar( $index ) ) {
			$widgets_in_area++;
		}
	}

	if ( $widgets_in_area ) {
		$area_widget_num_class = $prefix . $widgets_in_area . $suffix;
	} else {
		return false;
	}

	if ( $echo ) {
		echo $area_widget_num_class;
	} else {
		return $area_widget_num_class;
	}
}

/*
 * tail_classを利用すると、パラメーターで指定された複数のウィジェットエリアの最後のウィジェットエリアにclass名を
 * 表示し、marginなどの調整を行うことができます。ただし、Cafe Miraggioテーマでは必要がなかったため利用していません。
 * パラメーターは下記の通り
 * $widgets        : 複数のウィジェットエリアのスラッグを配列で指定。register_sidebarで指定したidの値。
 * $current_widget : 判別するウィジェットエリアのスラッグ。
 * $class          : 表示するclass名。初期値は'tail'
 */
function tail_class( $widgets, $current_widget, $class = 'tail' ) {
	if ( ! is_array( $widgets ) || ! count( $widgets ) ) {
		return false;
	}

	foreach ( $widgets as $index ) {
		if ( is_active_sidebar( $index ) ) {
			$tail = $index;
		}
	}
	
	if ( $tail == $current_widget ) {
		echo ' ' . esc_attr( $class );
	}
}

/*
 * リンクページを簡単に作成するショートコード用の関数を定義しています。
 * ショートコードは、本文内において動的な処理や複雑なソースコードを簡単に埋め込むことができる便利な機能です。
 * Cafe Miraggioでは、Linkページで利用されており、リンクメニューに登録されているWebサイトを本文内にリストで表示します。
 * Linkページの本文の記述を見ればわかりますが、ショートコードは、[bookmarks]といったブラケット囲みの記述がPHPで解釈され
 * HTMLに置き換わるものです。
 * WordPressに備わっている garelly や Contact form 7でフォームを表示するために記述する
 * [contact-form-7 id="499" title="コンタクトフォーム 1"] などもショートコードの1つです。
 * ショートコードの詳細については、ショートコード API（ http://wpdocs.sourceforge.jp/%E3%82%B7%E3%83%A7%E3%83%BC%E3%83%88%E3%82%B3%E3%83%BC%E3%83%89_API ）
 * を参照してください。
 */
function bookmark_shortcode( $attr ) {
	$defaults = array(
		'orderby' => 'name',
		'order' => 'ASC',
		'limit' => -1,
		'category' => '',
		'category_name' => '',
		'hide_invisible' => 1,
		'show_updated' => 0,
		'show_description' => 1,
		'show_images' => 1,
		'show_rating' => 0,
		'hide_invisible' => 1,
		'categorize' => 1,
		'title_li' => '',
		'title_before' => '<h2>',
		'title_after' => '</h2>',
		'category_orderby' => 'name',
		'category_order' => 'ASC',
		'class' => 'linkcat',
		'category_before' => '',
		'category_after' => '',
		'show_private ' => 0,
		'include' => '',
		'exclude' => '',
	);
	$r = shortcode_atts( $defaults, $attr );
	$r['echo'] = false;
	$r['title_before'] = html_entity_decode( $r['title_before'] );
	$r['title_after'] = html_entity_decode( $r['title_after'] );
	return wp_list_bookmarks( $r );
}
/*
 * bookmarksショートコードの登録を行います。
 * 最初のパラメーターがショートコード名、2つ目が実行される関数名となります。
 */
add_shortcode( 'bookmarks', 'bookmark_shortcode' );


/*
 * ナビゲーションメニューに登録されているメニュー数に応じたCSSのクラスを追加する add_nav_menu_num_class を定義します。
 * 出力されるclass名は、menu-items-num-*（*はメニュー数の数字）となり、cssで自動的に等幅のメニュー表示を行うことができるようになります。
 */
function add_nav_menu_num_class( $args ) {

	$menu = wp_get_nav_menu_object( $args['menu'] );

	// Get the nav menu based on the theme_location
	if ( ! $menu && $args['theme_location'] && ( $locations = get_nav_menu_locations() ) && isset( $locations[$args['theme_location']] ) ) {
		$menu = wp_get_nav_menu_object( $locations[$args['theme_location']] );
	}

	// get the first menu that has items if we still can't find a menu
	if ( ! $menu && ! $args['theme_location'] ) {
		$menus = wp_get_nav_menus();
		foreach ( $menus as $menu_maybe ) {
			if ( $menu_items = wp_get_nav_menu_items( $menu_maybe->term_id ) ) {
				$menu = $menu_maybe;
				break;
			}
		}
	}

	if ( $menu && ! is_wp_error( $menu ) && ! isset( $menu_items ) ) {
		$menu_items = wp_get_nav_menu_items( $menu->term_id );
	}

	if ( ! empty( $menu_items ) ) {
		$count = 0;
		foreach ( $menu_items as $menu_item ) {
			if ( $menu_item->menu_item_parent == '0' ) {
				$count++;
			}
		}

		$class = 'menu-items-num-' . $count;

		if ( $args['container'] ) {
			if ( $args['container_class'] ) {
				$args['container_class'] .= ' ' . $class;
			} else {
				$args['container_class'] = $class;
			}
		} else {
			if ( $args['menu_class'] ) {
				$args['menu_class'] .= ' ' . $class;
			} else {
				$args['menu_class'] = $class;
			}
		}
	} elseif ( ( ! $menu || is_wp_error( $menu ) || ( isset( $menu_items ) && empty( $menu_items ) && ! $args['theme_location'] ) ) && $args['fallback_cb'] == 'wp_page_menu' ) {
		$count = 0;
		$defaults = array('sort_column' => 'menu_order, post_title', 'menu_class' => 'menu', 'echo' => true, 'link_before' => '', 'link_after' => '');
		$list_args = wp_parse_args( $args, $defaults );
		$list_args = apply_filters( 'wp_page_menu_args', $list_args );
		
		if ( ! empty( $list_args['show_home'] ) ) {
			if ( get_option( 'show_on_front' ) == 'page' ) {
				if ( ! empty( $list_args['exclude'] ) ) {
					$list_args['exclude'] .= ',';
				} else {
					$list_args['exclude'] = '';
				}
				$list_args['exclude'] .= get_option( 'page_on_front' );
			}
			$count = 1;
		}
		
		$defaults = array(
			'depth' => 0, 'show_date' => '',
			'date_format' => get_option('date_format'),
			'child_of' => 0, 'exclude' => '',
			'title_li' => __('Pages'), 'echo' => 1,
			'authors' => '', 'sort_column' => 'menu_order, post_title',
			'link_before' => '', 'link_after' => '', 'walker' => '',
		);
		$list_args = wp_parse_args( $list_args, $defaults );
		$list_args['exclude'] = preg_replace( '/[^0-9,]/', '', $list_args['exclude'] );
		$exclude_array = ( $list_args['exclude'] ) ? explode(',', $list_args['exclude']) : array();
		$list_args['exclude'] = implode( ',', apply_filters( 'wp_list_pages_excludes', $exclude_array ) );

		// Query pages.
		$list_args['hierarchical'] = 0;
		$pages = get_pages( $list_args );
		if ( $pages ) {
			foreach ( $pages as $page ) {
				if ( $page->post_parent === 0 ) {
					$count++;
				}
			}
		}

		if ( $args['menu_class'] ) {
			$args['menu_class'] .= ' ' . 'menu-items-num-' . $count;
		} else {
			$args['menu_class'] = 'menu-items-num-' . $count;
		}
	}

	return $args;
}
add_filter( 'wp_nav_menu_args', 'add_nav_menu_num_class' );

/*
 * カスタムテンプレートタグ get_default_image を定義します。
 * default_image というフィルターフックを備えているため、子テーマを利用する際など、本テーマのファイルを上書きすることなく
 * 表示する画像を変更することも可能となっています。
 */
function get_default_image( $file ) {
	return apply_filters( 'default_image', get_template_directory_uri() . '/img/' . $file, $file );
}

/*
 * アーカイブページで見出しなどに利用できるカスタムテンプレートタグの single_archive_title を定義します。
 * 通常は、テンプレート内で複雑な分岐処理を行うか、テンプレートを分けて個別に表示することになりますが、single_archive_titleを
 * 利用すれば、面倒な分岐処理を割愛し、テンプレートの記述をシンプルにすることが出来ます。
 * パラメーターは、$echoが指定可能で、表示するか文字列として取得するかになります。
 */
function single_archive_title( $echo = true ) {
	if ( is_category() ) {
		$title = apply_filters( 'miraggio_search_title', sprintf( '%s', single_cat_title( '', false ) ) );
	} elseif ( is_tag() ) {
		$title = apply_filters( 'miraggio_tag_title', sprintf( '%s', single_tag_title( '', false ) ) );
	} elseif ( is_date() ) {
		if ( $m = get_query_var( 'm' ) ) {
			$year = substr( $m, 0, 4 );
			$title = apply_filters( 'miraggio_year_title', sprintf( '%s年', $year ) );
			if ( is_month() ) {
				$month = substr( $m, -1, 2 );
				$title .= apply_filters( 'miraggio_month_title', sprintf( '%s月', $month ) );
			}
		} else {
			$title = apply_filters( 'miraggio_year_title', sprintf( '%s年', get_query_var( 'year' ) ) );
			if ( $month = get_query_var( 'monthnum' ) ) {
				$title .= apply_filters( 'miraggio_month_title', sprintf( '%s月', $month ) );
				if ( $day = get_query_var( 'day' ) ) {
					$title .= apply_filters( 'miraggio_day_title', sprintf( '%s日', $day ) );
				}
			}
		}
	} elseif ( is_search() ) {
		$title = apply_filters( 'miraggio_search_title', sprintf( '『%s』の検索結果', get_search_query() ) );
	} elseif ( is_author() ) {
		$author = get_userdata( get_query_var( 'author' ) );
		$title = apply_filters( 'miraggio_author_title', sprintf( '%s', $author->display_name ) );
	} elseif ( is_post_type_archive() ) {
		$title = apply_filters( 'miraggio_post_type_title', sprintf( '%s', post_type_archive_title( '', false ) ) );
		} elseif ( is_tax() ) {
		$title = apply_filters( 'miraggio_tax_title', sprintf( '%s', single_term_title( '', false ) ) );
	}

	$title = apply_filters( 'single_archive_title', $title );

	if ( $echo ) {
		echo $title;
	} else {
		return $title;
	}
}

/*
 * 複数のウィジェットエリアを通じてウィジェットの表示が行われるかの判別をする
 * カスタムテンプレートタグの is_active_sidebar_area を定義します。
 * パラメーターは、判別するウィジェットエリアのスラッグを配列で指定します。
 */
function is_active_sidebar_area( $areas = array() ) {
	if ( is_array( $areas ) && ! empty( $areas ) ) {
		foreach ( $areas as $area ) {
			if ( is_active_sidebar( $area ) ) { return true; }
		}
	}
	return false;
}
